const hx = require("hbuilderx")
const WorkspaceEdit = hx.WorkspaceEdit;
const TextEdit = hx.TextEdit;
const eslint = require("eslint");
const fs = require("fs");
const diff = require("./diff.js");
let baseConfig = require("../.eslintrc.js");
const pluginName = require("../package.json").name;
const nls = require("hxnls");
const localize =  nls.loadMessageBundle(__filename);
function resolveSettings(doc,autofix) {
    let userConfigFile = hx.env.appData + "/extensions/eslint-js/.eslintrc.js";
    if(fs.existsSync(userConfigFile)){
        delete require.cache[require.resolve(userConfigFile)];
        baseConfig = require(userConfigFile);
    }
	//TODO 这里应该用workspace.getWorkspaceFolder API
	let workspaceFolder = doc.workspaceFolder;
	let contents = doc.getText();
	if(contents.length > 512 * 1024 * 1024){//如果大于512kb，则禁用autofix
		autofix = false;
	}
	let supportLanguages = ["javascript","javascript_es6"];
	let CLIEngine = eslint.CLIEngine;
	let cwd = process.cwd();
    let ESlint = eslint.ESlint;
	
	if (workspaceFolder && workspaceFolder.uri) {
		let folderUri = workspaceFolder.uri;
        let folderPath = (typeof folderUri === "string") ? folderUri : folderUri.fsPath;
		let userEslint = folderPath + "/node_modules/eslint";
		if (fs.existsSync(userEslint)) {
			CLIEngine = require(userEslint).CLIEngine;
            ESlint = require(userEslint).ESLint;
			if (CLIEngine === undefined) {
                if(ESlint)
                {
                   cwd = folderPath;
                   let enableHTMLScriptValidate = fs.existsSync(userEslint + "-plugin-html");
                   if(!enableHTMLScriptValidate){
                       baseConfig.plugins = [];
                   }
                }
                else{
                    let text = localize(0, null);
                    hx.window.showErrorMessage(text);
                    CLIEngine = eslint.CLIEngine;
                }
			} else {
				cwd = folderPath;
				let enableHTMLScriptValidate = fs.existsSync(userEslint + "-plugin-html");
				if(!enableHTMLScriptValidate){
					baseConfig.plugins = [];
				}
			}
		}
	}
	if(baseConfig.plugins.indexOf("html") > -1){
		supportLanguages.push("html");
		supportLanguages.push("html_es6");
	}
	return {
		library: {
			CLIEngine: CLIEngine
		},
        ESlint:ESlint,
		workingDirectory: cwd,
		options: {
			baseConfig: baseConfig,
			fix: autofix
		},
		supportLanguages:supportLanguages
	}
}
function withCLIEngine(callback, settings) {
	const cwd = process.cwd();
	const newOptions = settings.options;
	try {
		if (settings.workingDirectory) {
			newOptions.cwd = settings.workingDirectory;
		}
		const cli = new settings.library.CLIEngine(newOptions);
		return callback(cli);
	} catch(e){
		hx.window.clearStatusBarMessage();
		let output = hx.window.createOutputChannel(pluginName);
		let text = localize(1,null,pluginName);
		output.appendLine(text + e.message);
        
		if (cwd != process.cwd()) {
			process.chdir(cwd);
		}
	}
}
function withESlint(callback, settings) {
    const cwd = process.cwd();
	const newOptions = settings.options;
	try {
		if (settings.workingDirectory) {
			newOptions.cwd = settings.workingDirectory;
		}
		const eslint = new settings.ESlint(newOptions);
		return callback(eslint);
	} catch(e){
		hx.window.clearStatusBarMessage();
		let output = hx.window.createOutputChannel(pluginName);
		let text = localize(2,null,pluginName);
		output.appendLine(text + e.message);
        
		if (cwd != process.cwd()) {
			process.chdir(cwd);
		}
	}
}
function validateAndFixDocument(doc,autofix) {
	let text = doc.getText();
	let settings = resolveSettings(doc,autofix);
	if(settings.supportLanguages.indexOf(doc.languageId) < 0){
		return ;
	}
	let text1 = localize(3,null,pluginName)
	hx.window.setStatusBarMessage(text1);
    if(settings.ESlint)
    {
        withESlint(async function(eslint){
            let options = {
                filePath:doc.fileName,
                warnIgnored:true
            };
            let results = await eslint.lintText(text,options);
            let errorMsgs = [];
            if(results.length > 0)
            {
                for (let i = 0; i < results.length; i++) 
                {
                    let result = results[i];
                    if(result.errorCount > 0)
                     {
                        let messages = result.messages;
                        if (messages && messages.length > 0) {
                            for (let j = 0; j < messages.length; j++) {
                                let errMsg = messages[j];
                                if (errMsg.severity == 2) {
                                    errorMsgs.push({
                                        column: errMsg.column - 1,
                                        line: errMsg.line - 1,
                                        message: errMsg.message
                                    });
                                }
                            }
                        }
                     }
                     
                     if(result.output !== undefined) {
                         const diffs = diff.stringDiff(text, result.output, false);
                         let workspaceEdit = new WorkspaceEdit();
                         let edits = [];
                         for (let df of diffs) {
                             edits.push(new TextEdit({
                                 start: df.originalStart,
                                 end: df.originalStart + df.originalLength,
                                 }, result.output.substr(df.modifiedStart, df.modifiedLength)));
                             }
                             
                             workspaceEdit.set(doc.uri, edits);
                             hx.workspace.applyEdit(workspaceEdit);
                     }
                }
                
                let diagCollection = hx.languages.createDiagnosticCollection(pluginName);
                diagCollection.set(doc.uri, errorMsgs);
            }
            
            let level = "info";
            let text2 = localize(4,null, pluginName, errorMsgs.length);
            let text3 = localize(5,null);
            let statusMsg = text2;
            let showtime = 5*1000;
            if (errorMsgs.length > 0) {
                level = "error";
                statusMsg += text3;
                showtime = 30*1000;
            }
            
            hx.window.setStatusBarMessage(statusMsg, showtime, level);
            
        },settings);
        return;
    }
	withCLIEngine(function(cli) {
		var report = cli.executeOnText(text, doc.fileName);
		let errorMsgs = [];
		if (report && report.results && Array.isArray(report.results) && report.results.length == 1) {
			let result = report.results[0];
			let messages = result.messages;
			if (messages && messages.length > 0) {
				for (let j = 0; j < messages.length; j++) {
					let errorMsg = messages[j];
					if (errorMsg.severity == 2) {
						errorMsgs.push({
							column: errorMsg.column - 1,
							line: errorMsg.line - 1,
							message: errorMsg.message
						});
					}
				}
			}
			let diagCollection = hx.languages.createDiagnosticCollection(pluginName);
			diagCollection.set(doc.uri, errorMsgs);
			//自动修复
			if (result.output !== undefined) {
				const diffs = diff.stringDiff(text, result.output, false);
				let workspaceEdit = new WorkspaceEdit();
				let edits = [];
				for (let df of diffs) {
					edits.push(new TextEdit({
						start: df.originalStart,
						end: df.originalStart + df.originalLength
					}, result.output.substr(df.modifiedStart, df.modifiedLength)));
				}
				workspaceEdit.set(doc.uri, edits);
				hx.workspace.applyEdit(workspaceEdit);
			}
		}
		let level = "info";
		let text2 = localize(6,null, pluginName, errorMsgs.length);
		let text3 = localize(7,null);
		let statusMsg = text2;
		let showtime = 5*1000;
		if (errorMsgs.length > 0) {
			level = "error";
			statusMsg += text3;
			showtime = 30*1000;
		}
		hx.window.setStatusBarMessage(statusMsg, showtime, level);
	}, settings)
}
function activate(context) {
	let autoFixCommandDispose = hx.commands.registerCommand("extension.eslint.autoFix", function(context) {
		let editorPromise = hx.window.getActiveTextEditor();
		editorPromise.then((editor)=>{
			let doc = editor.document;
			validateAndFixDocument(doc,false);
		});
	});
    let eslintConfig = hx.workspace.getConfiguration("eslint-js")
    //这个是执行保存格式化的时候用的
    let willSaveTextDocumentEventDispose = hx.workspace.onWillSaveTextDocument(function(event){
        let autoFixOnSave = eslintConfig.get("autoFixOnSave",true);
		validateAndFixDocument(event.document,autoFixOnSave);
    });
    let didChangeTextDocumentEventDispose = hx.workspace.onDidChangeTextDocument(function(event){
        let validateOnDocumentChanged = eslintConfig.get("validateOnDocumentChanged");
		if(validateOnDocumentChanged){
			validateAndFixDocument(event.document,false);
		}
    });
    context.subscriptions.push(autoFixCommandDispose);
    context.subscriptions.push(willSaveTextDocumentEventDispose);
    context.subscriptions.push(didChangeTextDocumentEventDispose);
}
module.exports = {
	activate: activate
}
